@@ -52,10 +52,9 @@ This module provides Manager, a base class for managers.
 """
 
 from oslo_service import periodic_task
-import six
 
 import nova.conf
-from nova.db import base
+import nova.db.main.api
 from nova import profiler
 from nova import rpc
 
@@ -87,11 +86,10 @@ class ManagerMeta(profiler.get_traced_meta(), type(PeriodicTasks)):
     """
 
 
-@six.add_metaclass(ManagerMeta)
-class Manager(base.Base, PeriodicTasks):
+class Manager(PeriodicTasks, metaclass=ManagerMeta):
     __trace_args__ = {"name": "rpc"}
 
-    def __init__(self, host=None, db_driver=None, service_name='undefined'):
+    def __init__(self, host=None, service_name='undefined'):
         if not host:
             host = CONF.host
         self.host = host
@@ -99,18 +97,21 @@ class Manager(base.Base, PeriodicTasks):
         self.service_name = service_name
         self.notifier = rpc.get_notifier(self.service_name, self.host)
         self.additional_endpoints = []
-        super(Manager, self).__init__(db_driver)
+        super(Manager, self).__init__()
 
     def periodic_tasks(self, context, raise_on_error=False):
         """Tasks to be run at a periodic interval."""
         return self.run_periodic_tasks(context, raise_on_error=raise_on_error)
 
-    def init_host(self):
+    def init_host(self, service_ref):
         """Hook to do additional manager initialization when one requests
         the service be started.  This is called before any service record
-        is created.
+        is created, but if one already exists for this service, it is
+        provided.
 
         Child classes should override this method.
+
+        :param service_ref: An objects.Service if one exists, else None.
         """
         pass
 
