@@ -19,12 +19,12 @@ import re
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_policy import opts
 from oslo_policy import policy
 from oslo_utils import excutils
 
 
 from nova import exception
-from nova.i18n import _LE, _LW
 from nova import policies
 
 
@@ -41,6 +41,16 @@ USER_BASED_RESOURCES = ['os-keypairs']
 saved_file_rules = []
 KEY_EXPR = re.compile(r'%\((\w+)\)s')
 
+# TODO(gmann): Remove overriding the default value of config options
+# 'policy_file', 'enforce_scope', and 'enforce_new_defaults' once
+# oslo_policy change their default value to what is overridden here.
+DEFAULT_POLICY_FILE = 'policy.yaml'
+opts.set_defaults(
+    cfg.CONF,
+    DEFAULT_POLICY_FILE,
+    enforce_scope=True,
+    enforce_new_defaults=True)
+
 
 def reset():
     global _ENFORCER
@@ -68,11 +78,12 @@ def init(policy_file=None, rules=None, default_rule=None, use_conf=True,
     global saved_file_rules
 
     if not _ENFORCER:
-        _ENFORCER = policy.Enforcer(CONF,
-                                    policy_file=policy_file,
-                                    rules=rules,
-                                    default_rule=default_rule,
-                                    use_conf=use_conf)
+        _ENFORCER = policy.Enforcer(
+            CONF,
+            policy_file=policy_file,
+            rules=rules,
+            default_rule=default_rule,
+            use_conf=use_conf)
         # NOTE(gmann): Explictly disable the warnings for policies
         # changing their default check_str. During policy-defaults-refresh
         # work, all the policy defaults have been changed and warning for
@@ -115,10 +126,12 @@ def _warning_for_deprecated_user_based_rules(rules):
                 if resource in rule[0]]:
             continue
         if 'user_id' in KEY_EXPR.findall(rule[1]):
-            LOG.warning(_LW("The user_id attribute isn't supported in the "
-                            "rule '%s'. All the user_id based policy "
-                            "enforcement will be removed in the "
-                            "future."), rule[0])
+            LOG.warning(
+                "The user_id attribute isn't supported in the rule '%s'. "
+                "All the user_id based policy enforcement will be removed in "
+                "the future.",
+                rule[0]
+            )
 
 
 def set_rules(rules, overwrite=True, use_conf=False):
@@ -178,7 +191,7 @@ def authorize(context, action, target=None, do_raise=True, exc=None):
                                      do_raise=do_raise, exc=exc, action=action)
     except policy.PolicyNotRegistered:
         with excutils.save_and_reraise_exception():
-            LOG.exception(_LE('Policy not registered'))
+            LOG.exception('Policy not registered')
     except policy.InvalidScope:
         LOG.debug('Policy check for %(action)s failed with scope check '
                   '%(credentials)s',
