@@ -15,9 +15,10 @@
 
 import copy
 import io
+from unittest import mock
 
 import fixtures
-import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import conf
 from nova.tests import fixtures as nova_fixtures
@@ -42,7 +43,6 @@ class ServersTestBase(integrated_helpers._IntegratedTestBase):
         super(ServersTestBase, self).setUp()
 
         self.useFixture(nova_fixtures.LibvirtImageBackendFixture())
-        self.useFixture(nova_fixtures.CGroupsFixture())
         self.libvirt = self.useFixture(nova_fixtures.LibvirtFixture())
         self.useFixture(nova_fixtures.OSBrickFixture())
 
@@ -178,7 +178,12 @@ class ServersTestBase(integrated_helpers._IntegratedTestBase):
         self.assertNotIn(hostname, self.computes)
         self.assertNotIn(hostname, self.compute_rp_uuids)
 
-        self.computes[hostname] = _start_compute(hostname, host_info)
+        with mock.patch('nova.virt.node.get_local_node_uuid') as m:
+            m.return_value = str(getattr(uuids, 'node_%s' % hostname))
+            self.computes[hostname] = _start_compute(hostname, host_info)
+            # We need to trigger libvirt.Host() to capture the node-local
+            # uuid while we have it mocked out.
+            self.computes[hostname].driver._host.get_node_uuid()
 
         self.compute_rp_uuids[hostname] = self.placement.get(
             '/resource_providers?name=%s' % hostname).body[
