@@ -13,10 +13,10 @@
 # under the License.
 
 import collections
-import fixtures
-import mock
 import os.path
+from unittest import mock
 
+import fixtures
 from oslo_utils import fileutils
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import units
@@ -415,7 +415,9 @@ class _LibvirtEvacuateTest(integrated_helpers.InstanceHelperMixin):
 
         with mock.patch.object(fakelibvirt.Connection, 'getHostname',
                                return_value=name):
-            compute = self.start_service('compute', host=name)
+            with mock.patch('nova.virt.node.get_local_node_uuid') as m:
+                m.return_value = str(getattr(uuids, 'node_%s' % name))
+                compute = self.start_service('compute', host=name)
 
         compute.driver._host.get_connection().getHostname = lambda: name
         return compute
@@ -427,7 +429,6 @@ class _LibvirtEvacuateTest(integrated_helpers.InstanceHelperMixin):
         self.useFixture(nova_fixtures.NeutronFixture(self))
         self.useFixture(nova_fixtures.GlanceFixture(self))
         self.useFixture(func_fixtures.PlacementFixture())
-        self.useFixture(nova_fixtures.CGroupsFixture())
         fake_network.set_stub_network_methods(self)
 
         api_fixture = self.useFixture(
