@@ -11,7 +11,7 @@
 #    under the License.
 
 import copy
-import mock
+from unittest import mock
 
 from oslo_utils.fixture import uuidsentinel as uuids
 
